@@ -101,12 +101,24 @@ class TestPath(unittest.TestCase):
             assert g.is_dir()
             c, f, d = b.iterdir()
             assert c.is_file() and f.is_file()
-            e, = d.iterdir()
+            (e,) = d.iterdir()
             assert e.is_file()
-            h, = g.iterdir()
-            i, = h.iterdir()
+            (h,) = g.iterdir()
+            (i,) = h.iterdir()
             assert i.is_file()
 
+    def test_is_file_missing(self):
+        for alpharep in self.zipfile_alpharep():
+            root = zipp.Path(alpharep)
+            assert not root.joinpath('missing.txt').is_file()
+
+    def test_iterdir_on_file(self):
+        for alpharep in self.zipfile_alpharep():
+            root = zipp.Path(alpharep)
+            a, b, g = root.iterdir()
+            with self.assertRaises(ValueError):
+                a.iterdir()
+
     def test_subdir_is_dir(self):
         for alpharep in self.zipfile_alpharep():
             root = zipp.Path(alpharep)
@@ -142,6 +154,14 @@ class TestPath(unittest.TestCase):
         with self.assertRaises(IsADirectoryError):
             zf.joinpath('b').open()
 
+    def test_open_binary_invalid_args(self):
+        for alpharep in self.zipfile_alpharep():
+            root = zipp.Path(alpharep)
+            with self.assertRaises(ValueError):
+                root.joinpath('a.txt').open('rb', encoding='utf-8')
+            with self.assertRaises(ValueError):
+                root.joinpath('a.txt').open('rb', 'utf-8')
+
     def test_open_missing_directory(self):
         """
         Attempting to open a missing directory raises FileNotFoundError.
@@ -160,7 +180,7 @@ class TestPath(unittest.TestCase):
     def test_joinpath(self):
         for alpharep in self.zipfile_alpharep():
             root = zipp.Path(alpharep)
-            a = root.joinpath("a")
+            a = root.joinpath("a.txt")
             assert a.is_file()
             e = root.joinpath("b").joinpath("d").joinpath("e.txt")
             assert e.read_text() == "content of e"
@@ -168,7 +188,7 @@ class TestPath(unittest.TestCase):
     def test_traverse_truediv(self):
         for alpharep in self.zipfile_alpharep():
             root = zipp.Path(alpharep)
-            a = root / "a"
+            a = root / "a.txt"
             assert a.is_file()
             e = root / "b" / "d" / "e.txt"
             assert e.read_text() == "content of e"
@@ -226,11 +246,9 @@ class TestPath(unittest.TestCase):
             a, b, g = root.iterdir()
             alpharep.writestr('foo.txt', 'foo')
             alpharep.writestr('bar/baz.txt', 'baz')
-            assert any(
-                child.name == 'foo.txt'
-                for child in root.iterdir())
+            assert any(child.name == 'foo.txt' for child in root.iterdir())
             assert (root / 'foo.txt').read_text() == 'foo'
-            baz, = (root / 'bar').iterdir()
+            (baz,) = (root / 'bar').iterdir()
             assert baz.read_text() == 'baz'
 
     HUGE_ZIPFILE_NUM_ENTRIES = 2 ** 13
@@ -259,3 +277,17 @@ class TestPath(unittest.TestCase):
     def test_implied_dirs_performance(self):
         data = ['/'.join(string.ascii_lowercase + str(n)) for n in range(10000)]
         zipp.CompleteDirs._implied_dirs(data)
+
+    def test_read_does_not_close(self):
+        for alpharep in self.zipfile_ondisk():
+            with zipfile.ZipFile(alpharep) as file:
+                for rep in range(2):
+                    zipp.Path(file, 'a.txt').read_text()
+
+    def test_subclass(self):
+        class Subclass(zipp.Path):
+            pass
+
+        for alpharep in self.zipfile_alpharep():
+            root = Subclass(alpharep)
+            assert isinstance(root / 'b', Subclass)
